package vsp.uloha14;

import java.util.Random;

/**
 * Trida pro generovani nahodnych cisel a vypoctu statistik
 * @author Jiri Kucera (kalwi@students.zcu.cz)
 */
public class Statistics {
    private static Random random;
    
    private static final int H_SIZE = 10;	// pocet sloupcu histogramu
    private static final int H_COL_SIZE = 20;	// maximalni vyska sloupce histogramu

    private double expectedValueE = 0;
    private double varianceE = 0;
    
    private double expectedValueT = 0;
    private double varianceT = 0;

    private int[] histogram;
    private double[] histogramLegend;

    private double hMax;			// hranice, nad niz budou vygenerovana cisla ignorovana

    /**
     * Konstruktor vytvori objekt pro generovani cisel a vypocet statistik.
     * @param count pocet generovanych cisel
     * @param lambda parametr lambda exponencialniho rozdeleni
     */
    public Statistics(int count, double lambda) {
	
	random = new Random();

	expectedValueT = 1. / lambda;
	varianceT = 1. / (lambda * lambda);

	hMax = expectedValueT * 3;

	histogram = new int[H_SIZE];
	histogramLegend = new double[H_SIZE];

	for (int i = 0; i < H_SIZE; i++) {
	    histogramLegend[i] = (i + 1) * hMax / (double) H_SIZE;
	}

	int highest = 1;
	double sum = 0;
	double sum2 = 0;

	// generovani cisel
	for (int i = 0; i < count; i++) {
	    
	    double number = generateRandom(lambda);

	    sum += number / count;
	    sum2 += number * number / count;

	    // pridani do histogramu
	    int hIndex = (int) (number * H_SIZE / hMax);

	    // cisla mimo rozsah histogramu se ignoruji
	    if (hIndex < H_SIZE) {
		histogram[hIndex]++;
		if (histogram[hIndex] > highest) {
		    highest = histogram[hIndex];
		}
	    }
	}

	// stredni hodnota
	expectedValueE = sum;

	// rozptyl
	varianceE = sum2 - expectedValueE * expectedValueE;

	// normalizace histogramu
	for (int i = 0; i < H_SIZE; i++) {
	    histogram[i] = histogram[i] * H_COL_SIZE / highest;
	}

    }

    /**
     * @return Vypocitana stredni hodnota
     */
    public double getEvaluatedExpectedValue() {
	return expectedValueE;
    }

    /**
     * @return Teoreticka stredni hodnota
     */
    public double getTheoreticalExpectedValue() {
	return expectedValueT;
    }

    /**
     * @return Vypocitany rozptyl
     */
    public double getEvaluatedVariance() {
	return varianceE;
    }

    /**
     * @return Teoreticky rozptyl
     */
    public double getTheoreticalVariance() {
	return varianceT;
    }

    /**
     * @return Pole s histogramem
     */
    public int[] getHistogram() {
	return histogram;
    }

    /**
     * @return Pole s popisky sloupcu histogramu
     */
    public double[] getHistogramLegend() {
	return histogramLegend;
    }

    /**
     * Vygeneruje nahodne cislo s exponencialnim rozdelenim pravdepodobnosti.
     * @param lambda parametr lambda exponencialniho rozdeleni
     * @return vygenerovane nahodne cislo
     */
    public static double generateRandom(double lambda) {
	return - Math.log(1 - random.nextDouble()) / lambda;
    }

}
